@@ -13,6 +13,7 @@ import logging
 import random
 import re
 import os
+import sys
 from functools import reduce
 
 import argparse
@@ -48,6 +49,7 @@ class HWAnalysis(object):
         self.no_comb_and = False
         self.prob_comb = 1.0
         self.skip_print_res = False
+        self.log_prints = False
         self.do_only_top_comb = False
         self.do_only_top_deg = False
         self.no_term_map = False
@@ -197,7 +199,7 @@ class HWAnalysis(object):
             comb_subres = self.term_eval.new_buffer()
             hws_input = [0] * len(self.input_poly)
             for idx, poly in enumerate(self.input_poly):
-                obs_cnt = self.term_eval.hw(self.term_eval.eval_poly(poly, res=comb_res, subres=comb_subres))
+                obs_cnt = self.term_eval.eval_poly_hw(common.mutable_poly(poly), res=comb_res, subres=comb_subres)
                 hws_input[idx] = obs_cnt
                 self.input_poly_hws[idx] += obs_cnt
 
@@ -243,7 +245,10 @@ class HWAnalysis(object):
     def tprint(self, *args, **kwargs):
         if self.skip_print_res:
             return
-        print(*args, **kwargs)
+        if self.log_prints:
+            logger.info(*args, **kwargs)
+        else:
+            print(*args, **kwargs)
 
     def unrank(self, deg, index):
         """
@@ -548,10 +553,19 @@ class HWAnalysis(object):
         top_res.sort(key=lambda x: abs(x.zscore), reverse=True)
         return top_res
 
+    def comb_should_add(self, zscore, top_res):
+        if self.best_x_combinations is None:
+            return True
+        if len(top_res) <= self.best_x_combinations:
+            return True
+        return zscore > top_res[0][0]
+
     def comb_add_result(self, comb, top_res):
         """
         Adds result to the top results.
         Can use heap to optimize eval speed if caller does not require all results.
+        Should be called only if comb_should_add() returns true
+
         :param comb:
         :param top_res:
         :return:
@@ -566,7 +580,7 @@ class HWAnalysis(object):
         if len(top_res) <= self.best_x_combinations:
             heapq.heappush(top_res, new_item)
 
-        elif abs(comb.zscore) > top_res[0][0]:  # this difference is larger than minimum in heap
+        elif new_item[0] > top_res[0][0]:  # this difference is larger than minimum in heap
             heapq.heapreplace(top_res, new_item)
 
     def comb_base(self, top_comb_cur, top_terms, top_res, num_evals, poly_builder, ref_hws=None):
@@ -582,27 +596,77 @@ class HWAnalysis(object):
         :param ref_hws: reference results
         :return:
         """
-        for idx, places in enumerate(common.term_generator(top_comb_cur, len(top_terms) - 1, self.prob_comb)):
+        iterator = common.term_generator(top_comb_cur, len(top_terms) - 1, self.prob_comb)
+        for idx, places in enumerate(iterator):
             poly = poly_builder(places, top_terms)
             expp = self.term_eval.expp_poly(poly)
             exp_cnt = num_evals * expp
             if exp_cnt == 0:
                 continue
 
-            obs_cnt = self.term_eval.hw(self.term_eval.eval_poly(poly, res=self.comb_res, subres=self.comb_subres))
+            obs_cnt = self.term_eval.eval_poly_hw(poly, self.comb_res, self.comb_subres)
             zscore = common.zscore(obs_cnt, exp_cnt, num_evals)
+            if not self.comb_should_add(abs(zscore), top_res):
+                continue
 
             comb = None
             if ref_hws is None:
                 comb = Combined(poly, expp, exp_cnt, obs_cnt, zscore)
+
             else:
-                ref_obs_cnt = self.ref_term_eval.hw(
-                    self.ref_term_eval.eval_poly(poly, res=self.comb_res, subres=self.comb_subres))
+                ref_obs_cnt = self.ref_term_eval.eval_poly_hw(poly, res=self.comb_res, subres=self.comb_subres)
                 zscore_ref = common.zscore(ref_obs_cnt, exp_cnt, num_evals)
                 comb = Combined(poly, expp, exp_cnt, obs_cnt, zscore - zscore_ref)
 
             self.comb_add_result(comb, top_res)
 
+    def comb_base_batch(self, top_comb_cur, top_terms, top_res, num_evals, poly_builder, ref_hws=None):
+        """
+        Base skeleton for generating all combinations from top_terms up to degree top_comb_cur.
+        Evaluates polynomial, computes expected results, computes zscores.
+
+        :param top_comb_cur: current degree of the combination
+        :param top_terms: top terms buffer to choose terms out of
+        :param top_res: top results accumulator to put, Combined(poly, expp, exp_cnt, obs_cnt, zscore - zscore_ref)
+        :param num_evals: number of evaluations in this round - zscore computation
+        :param poly_builder: function of (places, top_terms) returns a new polynomial
+        :param ref_hws: reference results
+        :return:
+        """
+        iterator = common.term_generator(top_comb_cur, len(top_terms) - 1, self.prob_comb, clone=True)
+        btch = self.term_eval.base_buff_size
+        polys = [None] * btch
+        hws = [0] * btch
+
+        for batch in common.chunks(iterator, btch):
+            ln = len(batch)
+            for ix, places in enumerate(batch):
+                polys[ix] = poly_builder(places, top_terms)
+
+            obs_cnts = self.term_eval.eval_polys_hw(polys[:ln], res=self.comb_res, subres=self.comb_subres, hws=hws)
+            for ix in range(ln):
+                poly = polys[ix]
+                expp = self.term_eval.expp_poly(poly)
+                exp_cnt = num_evals * expp
+                if exp_cnt == 0:
+                    continue
+
+                obs_cnt = obs_cnts[ix]
+                zscore = common.zscore(obs_cnt, exp_cnt, num_evals)
+                if not self.comb_should_add(abs(zscore), top_res):
+                    continue
+
+                comb = None
+                if ref_hws is None:
+                    comb = Combined(poly, expp, exp_cnt, obs_cnt, zscore)
+
+                else:
+                    ref_obs_cnt = self.ref_term_eval.eval_poly_hw(poly, res=self.comb_res, subres=self.comb_subres)
+                    zscore_ref = common.zscore(ref_obs_cnt, exp_cnt, num_evals)
+                    comb = Combined(poly, expp, exp_cnt, obs_cnt, zscore - zscore_ref)
+
+                self.comb_add_result(comb, top_res)
+
     def comb_xor(self, top_comb_cur, top_terms, top_res, num_evals, ref_hws=None):
         """
         Combines top terms with XOR operation
@@ -698,6 +762,7 @@ class Booltest(object):
         self.input_poly = []
         self.input_objects = []
         self.dump_cpu_info = False
+        self.alpha = 1./40000
 
         self.hwanalysis = None
         self.timer_data_read = timer.Timer(start=False)
@@ -894,7 +959,7 @@ class Booltest(object):
             print('  Expected probability: %s' % expp)
 
     def try_find_refdb(self):
-        if self.args.ref_db:
+        if 'ref_db' in self.args and self.args.ref_db:
             return self.args.ref_db
 
         fname = 'pval_db.json'
@@ -918,8 +983,14 @@ class Booltest(object):
 
         return None
 
+    def check_ref_db(self, hwanalysis):
+        if not hwanalysis.ref_db_path:
+            hwanalysis.ref_db_path = self.try_find_refdb()
+        else:
+            logger.info("Ref DB path: %s" % hwanalysis.ref_db_path)
+
     def init_params(self):
-        self.blocklen = int(self.defset(self.args.blocklen, 128))
+        self.blocklen = int(self.defset(self.args.blocklen, 256))
 
         # Default params
         if self.args.default_params:
@@ -931,9 +1002,10 @@ class Booltest(object):
             self.args.topterm_heap = True
             self.args.topterm_heap_k = 256
 
-        self.deg = int(self.defset(self.args.degree, 3))
+        self.deg = int(self.defset(self.args.degree, 2))
         self.top_k = int(self.args.topk) if self.args.topk is not None else None
         self.top_comb = int(self.defset(self.args.combdeg, 2))
+        self.alpha = self.args.alpha
         self.all_deg = self.args.alldeg
         self.rounds = int(self.args.rounds) if self.args.rounds is not None else None
         self.do_halving = self.args.halving
@@ -962,6 +1034,7 @@ class Booltest(object):
         hwanalysis.best_x_combinations = self.args.best_x_combinations
         hwanalysis.ref_db_path = self.try_find_refdb()
         hwanalysis.comp_indiv_pval = self.args.indiv_pval
+        hwanalysis.log_prints = self.args.log_prints if (self.args and 'log_prints' in self.args) else False
 
         # compute classical analysis only if there are no input polynomials
         hwanalysis.all_deg_compute = len(self.input_poly) == 0
@@ -1012,6 +1085,86 @@ class Booltest(object):
             logger.info('Updating TV to %d' % tvsize)
         return int(tvsize)
 
+    def human_poly(self, poly):
+        p = []
+        for term in poly:
+            p.append(' * '.join(['x_{%d}' % int(x) for x in term]))
+        if len(p) > 1:
+            p = ['(%s)' % x for x in p]
+        return ' + '.join(p)
+
+    def print_human_readable_results(self, jsres):
+        if self.args and 'no_summary' in self.args and self.args.no_summary:
+            return
+        
+        try:
+            print('-' * 80)
+            print('BoolTest run with the configuration (%s, %s, %s)' % (jsres['blocklen'], jsres['degree'], jsres['comb_degree']))
+            print(' - the block size is %s, i.e., the number of variables in the polynomials. x_{0}, ..., x_{%s}' % (jsres['blocklen'], jsres['blocklen'] - 1))
+            print(' - the maximal order of the first level terms is %s. Combined by AND.' % (jsres['degree']))
+            print(' - the maximal order of the second level polynomials is %s. Combined by XOR/AND.' % (jsres['comb_degree']))
+            print('')
+            for ix, inp in enumerate(jsres['inputs']):
+                print('-' * 80)
+
+                dist = inp['res'][0]['dists'][0]
+                aux_dist = ' in the first half of the data' if self.do_halving else ''
+                first_half_rejected = False
+                print('[+] Results for input: %s' % inp['iobj'])
+                print('The best distinguisher found%s: %s' % (aux_dist, self.human_poly(dist['poly'])))
+                print(' - z-score achieved: %s' % abs(dist['zscore']))
+                print('')
+
+                if 'rejects' in inp['res'][0]:
+                    alpha = 1./inp['res'][0]['ref_samples']
+                    print('BoolTest has reference statistics for used configuration, number of collected samples: %s'
+                          % inp['res'][0]['ref_samples'])
+                    print('Allowed z-score region in the reference data: [%s, %s]' % inp['res'][0]['ref_minmax'])
+                    if inp['res'][0]['rejects']:
+                        first_half_rejected = True
+                        print('As the achieved z-score %s lies outside of the interval, the randomness hypothesis '
+                              'is REJECTED with alpha %.e' % (abs(dist['zscore']), alpha))
+                        print(' = Data is not random')
+                    else:
+                        print('As the achieved z-score %s lies in the allowed interval, the randomness hypothesis '
+                              'cannot be rejected with the alpha %.e' % (abs(dist['zscore']), alpha))
+                        print(' = BoolTest could not find statistically significant non-randomness')
+
+                else:
+                    print('Unfortunately, for selected BoolTest parameters we don\'t have reference statistics '
+                          'computed, thus we cannot say if the randomness hypothesis is rejected or not.')
+                    if not self.do_halving:
+                        print('You may try to run BoolTest with --halving parameter which runs a different evaluation '
+                              'test without a need for reference data computation')
+
+                    print('Expert tip: If you wish to analyze the input data with these parameters, '
+                          'you need to collect reference statistics. For that, run BoolTest on a large sample '
+                          'of random data (e.g., AES with random key and input), collect z-score statistics '
+                          'and find maximum and minimum. If measured z-score lies outside of this interval, '
+                          'you can reject the randomness hypothesis with alpha=1/num_of_reference_samples')
+
+                if not self.do_halving:
+                    continue
+
+                hlv = inp['res'][1]['halvings'][0]
+                print('\nHalving results (test on the another half of the input):')
+                print('The selected distinguisher from the first BoolTest run: %s' % self.human_poly(hlv['poly']))
+                print('The computed p-value on the input data: %.e' % hlv['pval'])
+                if hlv['pval'] < self.alpha:
+                    print('As the p-value is lower than alpha=%.e, the randomness hypothesis is REJECTED' % self.alpha)
+                    print(' = Data is not random')
+
+                else:
+                    print('As the p-value >= alpha %.e, the randomness hypothesis cannot be rejected' % self.alpha)
+                    print(' = BoolTest could not find statistically significant non-randomness')
+                    if first_half_rejected:
+                        print('However, the first phase of the BoolTest run on the first half of the input data '
+                              'found a significant statistical deviation so this can be considered as a weak fail. '
+                              'You may want to run the BoolTest without --halving to assess the whole data length.')
+
+        except Exception as e:
+            logger.error('Exception in processing the results: %s' % e, exc_info=e)
+
     def work(self):
         """
         Main entry point - data processing
@@ -1072,7 +1225,7 @@ class Booltest(object):
             jscres['offset'] = coffset
 
             tvsize = self.adjust_tvsize(tvsize, size, coffset)
-
+            logger.warning("FUCK")
             self.hwanalysis = self.setup_hwanalysis(self.deg, self.top_comb, self.top_k, self.all_deg, zscore_thresh)
             if self.hwanalysis.ref_db_path:
                 logger.info('Using reference data file %s' % self.hwanalysis.ref_db_path)
@@ -1093,6 +1246,12 @@ class Booltest(object):
             with iobj:
                 self.analyze_iobj(iobj, coffset, tvsize, jscres['res'])
 
+            for rr in jscres['res']:
+                if 'dists' in rr:
+                    rr['dists'] = [NoIndent(common.jsunwrap(x)) for x in rr['dists']]
+                if 'halvings' in rr:
+                    rr['halvings'] = [NoIndent(common.jsunwrap(x)) for x in rr['halvings']]
+
             logger.info('Finished processing: %s ' % iobj)
             logger.info('Data read: %s B' % iobj.data_read)
             logger.info('Read data SHA1:   %s ' % iobj.sha1.hexdigest())
@@ -1129,7 +1288,11 @@ class Booltest(object):
             with open(self.args.json_out_file, 'w+') as fh:
                 common.json_dump(jsout, fh, **kwargs)
 
-        return common.jsunwrap(jsout)
+        jsout = common.jsunwrap(jsout)
+
+        # Human-understandable results.
+        self.print_human_readable_results(jsout)
+        return jsout
 
     def analyze_iobj(self, iobj, coffset=0, tvsize=None, jscres=None):
         data_read = 0
@@ -1202,10 +1365,10 @@ class Booltest(object):
                     pval = stats.binom_test(cr.obs_cnt, n=ntrials, p=cr.expp, alternative='two-sided')
 
                     jsresc = collections.OrderedDict()
-                    jsresc['poly'] = self.noindent(common.immutable_poly(cr.poly))
                     jsresc['nsamples'] = ntrials
                     jsresc['nsucc'] = cr.obs_cnt
                     jsresc['pval'] = pval
+                    jsresc['poly'] = self.noindent(common.immutable_poly(cr.poly))
                     jsres['halvings'].append(jsresc)
 
                     logger.info(
@@ -1232,7 +1395,7 @@ class Booltest(object):
         return jscres
 
     def main(self):
-        logger.debug('App started')
+        logger.debug('App started, path: %s, version: %s' % (sys.path, sys.version_info))
 
         parser = self.argparser()
         self.args = parser.parse_args()
@@ -1247,10 +1410,10 @@ class Booltest(object):
         parser.add_argument('--ref-db', dest='ref_db',
                             help='Reference JSON database file')
 
-        parser.add_argument('--block', dest='blocklen',
+        parser.add_argument('--block', dest='blocklen', type=int, default=256,
                             help='block size in bits, number of bit variables to construct terms from')
 
-        parser.add_argument('--degree', dest='degree',
+        parser.add_argument('--degree', dest='degree', type=int, default=2,
                             help='maximum degree of terms to construct in the first phase (AND)')
 
         parser.add_argument('--tv', dest='tvsize',
@@ -1315,7 +1478,7 @@ class Booltest(object):
         parser.add_argument('--topterm-heap', dest='topterm_heap', action='store_const', const=True, default=False,
                             help='Use heap to compute best K terms for stats & input to the combinations')
 
-        parser.add_argument('--topterm-heap-k', dest='topterm_heap_k', default=None, type=int,
+        parser.add_argument('--topterm-heap-k', dest='topterm_heap_k', default=256, type=int,
                             help='Number of terms to keep in the heap, should be at least top_k')
 
         parser.add_argument('--best-x-combs', dest='best_x_combinations', default=None, type=int,
@@ -1362,6 +1525,16 @@ class Booltest(object):
 
         parser.add_argument('--halving-top', dest='halving_top', type=int, default=1,
                             help='Number of top distinguishers to select to the halving phase')
+
+        parser.add_argument('--alpha', dest='alpha', type=float, default=1e-4,
+                            help='Alpha for pvalue hypothesis rejection, for halving method only')
+
+        parser.add_argument('--no-summary', dest='no_summary', action='store_const', const=True, default=False,
+                            help='Turns off the human readable summary')
+
+        parser.add_argument('--log-prints', dest='log_prints', action='store_const', const=True, default=False,
+                            help='Do not write info logs to stdout, log to stderr instead')
+
         return parser
 
 
